import { any, hasIn, is, path, pipe, split } from 'ramda';
var INDENTATION = 12;
var prop = pipe(split(/[[\].]/), path);
export var isListElement = function (val) {
    return is(Object, val) && !prop('key')(val);
};
export var isDictionaryElement = function (value) {
    return is(Object, value) &&
        hasIn('key', value) &&
        (is(String, prop('key')(value)) || is(Number, prop('key')(value)));
};
export var isMetaList = function (value) {
    return is(Array, value) && any(isListElement, value);
};
export var isMetaDictionary = function (value) {
    return is(Array, value) && any(isDictionaryElement, value);
};
var stringify = function (space, val, padding) {
    var indentation = ' '.repeat(padding);
    return JSON.stringify(val, null, space)
        .replace(/\\n/g, '\n')
        .replace(/}$/, indentation + '}')
        .replace(/]$/, indentation + ']');
};
var isJsonString = function (value) {
    try {
        var parsedValue = JSON.parse(value);
        return is(Object, parsedValue);
    }
    catch (e) {
        return false;
    }
};
var isBooleanString = function (value) {
    return value === 'true' || value === 'false';
};
var getQuotedString = function (value) {
    return isBooleanString(value) ? value : "\"" + value + "\"";
};
var getFormattedValue = function (value, offset) {
    return value
        .split(/\n/)
        .map(function (item, index) { return (index === 0 ? item : "" + offset + item); })
        .join('\n');
};
export function normalizeIssueVariable(keyValue, value, padding, indentation, withoutQuotes) {
    var _a;
    if (indentation === void 0) { indentation = INDENTATION; }
    if (is(String, value)) {
        if (isJsonString(String(value))) {
            return stringify(indentation, JSON.parse(String(value)), padding);
        }
        var elementPadding = ' '.repeat((withoutQuotes ? 3 : 5) + padding + keyValue.length);
        var formattedValue = getFormattedValue(value, elementPadding);
        return getQuotedString(formattedValue);
    }
    if (isMetaList(value) || isMetaDictionary(value)) {
        var offset_1 = ' '.repeat(indentation);
        var endOffset = ' '.repeat(padding);
        var items = value
            .filter(Boolean)
            .map(function (item) {
            if (is(String, item)) {
                return isJsonString(item)
                    ? JSON.stringify(JSON.parse(item), null, indentation)
                    : item;
            }
            var key = item.key, value = item.value;
            // Have to use `getFormattedValue` above `JSON.stringify` to shift the closing object bracket
            var formattedValue = isJsonString(String(value))
                ? getFormattedValue(JSON.stringify(JSON.parse(String(value)), null, indentation), offset_1)
                : getQuotedString(value);
            return key ? "\"" + key + "\": " + formattedValue : formattedValue;
        });
        var content = items.map(function (item) { return "" + offset_1 + item; }).join(',\n');
        return "[\n" + content + "\n" + endOffset + "]";
    }
    if (isDictionaryElement(value)) {
        return stringify(keyValue.length, (_a = {}, _a[value.key] = value.value, _a), padding);
    }
    if (isListElement(value)) {
        if (is(Array, value.value)) {
            return stringify(indentation, value.value, padding);
        }
        var listValue = String(value.value);
        return isJsonString(listValue)
            ? stringify(indentation, JSON.parse(listValue), padding)
            : getQuotedString(listValue);
    }
    return String(value);
}
